
/* ====================================================================
 * Copyright (c) 1996 The Apache Group.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by the Apache Group
 *    for use in the Apache HTTP server project (http://www.apache.org/)."
 *
 * 4. The names "Apache Server" and "Apache Group" must not be used to
 *    endorse or promote products derived from this software without
 *    prior written permission.
 *
 * 5. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by the Apache Group
 *    for use in the Apache HTTP server project (http://www.apache.org/)."
 *
 * THIS SOFTWARE IS PROVIDED BY THE APACHE GROUP ``AS IS'' AND ANY
 * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE APACHE GROUP OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Group and was originally based
 * on public domain software written at the National Center for
 * Supercomputing Applications, University of Illinois, Urbana-Champaign.
 * For more information on the Apache Group and the Apache HTTP server
 * project, please see <http://www.apache.org/>.
 *
 */


/*
**  mod_rewrite_status.c -- Rewriting Engine Status Module Code
**                       _                            _ _ 
**   _ __ ___   ___   __| |    _ __ _____      ___ __(_) |_ ___ 
**  | '_ ` _ \ / _ \ / _` |   | '__/ _ \ \ /\ / / '__| | __/ _ \
**  | | | | | | (_) | (_| |   | | |  __/\ V  V /| |  | | ||  __/
**  |_| |_| |_|\___/ \__,_|___|_|  \___| \_/\_/ |_|  |_|\__\___|
**                       |_____|
**
**  URL Rewriting Module, Version 2.3.0 (09-09-1996)
**
**  This module uses a rule-based rewriting engine (based on a
**  regular-expression parser) to rewrite requested URLs on the fly. 
**  
**  It supports an unlimited number of additional rule conditions (which can
**  operate on a lot of variables, even on HTTP headers) for granular
**  matching and even external database lookups (either via plain text
**  tables, DBM hash files or even external processes) for advanced URL
**  substitution.
**  
**  It operates on the full URLs (including the PATH_INFO part) both in
**  per-server context (httpd.conf) and per-dir context (.htaccess) and even
**  can generate QUERY_STRING parts on result.   The rewriting result finally
**  can lead to internal subprocessing, external request redirection or even
**  to internal proxy throughput.
**
**  The documentation and latest release can be found on
**  http://www.engelschall.com/sw/mod_rewrite/
**
**  Copyright (c) 1996 The Apache Group,    All rights reserved.
**  Copyright (c) 1996 Ralf S. Engelschall, All rights reserved.
**
**  Written for The Apache Group by
**      Ralf S. Engelschall
**      rse@engelschall.com
**      http://www.engelschall.com/~rse
*/




    /* from the underlaying Unix system ... */
#include <time.h>

    /* from the Apache server ... */
#include "httpd.h"
#include "http_config.h"
#include "http_core.h"
#include "http_protocol.h"

    /* now our own stuff ... */
#include "mod_rewrite.h"




/*
**
**  forward declarations
**
*/

    /* static config */
handler_rec rewrite_status_handlers[];

    /* import mod_rewrite config */
extern module rewrite_module;

    /* the handler */
static int rewrite_status_handler(request_rec *r);


/*
**
**  defines
**
*/

#define REWRITESTATUS_MAGIC_TYPE "application/x-httpd-rewritestatus"

    /* ID values for command table */
#define STAT_OPT_END            -1
#define STAT_OPT_REFRESH        0
#define STAT_OPT_AUTO           2

    /* option command table */
struct stat_opt {
    int id;
    char *form_data_str;
    char *hdr_out_str;
} options[] = { 
    { STAT_OPT_REFRESH, "refresh", "Refresh" },
    { STAT_OPT_AUTO,    "auto",    NULL      },
    { STAT_OPT_END,     NULL,      NULL      }
};


/*
**
**  our interface to the Apache server kernel
**
*/

module rewrite_status_module = {
   STANDARD_MODULE_STUFF,
   NULL,                        /* initializer */
   NULL,                        /* dir config creater */
   NULL,                        /* dir merger --- default is to override */
   NULL,                        /* server config */
   NULL,                        /* merge server config */
   NULL,                        /* command table */
   rewrite_status_handlers,     /* handlers */
   NULL,                        /* filename translation */
   NULL,                        /* check_user_id */
   NULL,                        /* check auth */
   NULL,                        /* check access */
   NULL,                        /* type_checker */
   NULL,                        /* fixups */
   NULL                         /* logger */
};

handler_rec rewrite_status_handlers[] =
{
    { REWRITESTATUS_MAGIC_TYPE, rewrite_status_handler },
    { "server-rewritestatus",   rewrite_status_handler },
    { NULL }
};




/*
**
**  The Handler itself
**
*/

static int rewrite_status_handler(request_rec *r)
{
    void *sconf;
    rewrite_server_conf *conf;
    array_header *rewriterules;
    rewriterule_entry *entries;
    rewriterule_entry *p;
    char flagstr[MAX_STRING_LEN];
    char notstr[MAX_STRING_LEN];
    char output[MAX_STRING_LEN];
    int i;
    int j;
    char *loc;
    time_t nowtime=time(NULL);
    time_t up_time;
    int res;
    int ready=0;
    int busy=0;
    int short_report=0;
    server_rec *server = r->server;

    if (r->method_number != M_GET)
        return NOT_IMPLEMENTED;
    r->content_type = "text/html";

    /*
     * Simple table-driven form data set parser that lets you alter the header
     */
    if (r->args)
    {
        i = 0;
        while (options[i].id != STAT_OPT_END)
        {
            if ((loc = strstr(r->args,options[i].form_data_str)) != NULL)
            {
                switch (options[i].id)
                {
                  case STAT_OPT_REFRESH:
                      if(*(loc + strlen(options[i].form_data_str)) == '=')
                          table_set(r->headers_out,options[i].hdr_out_str,
                            loc+strlen(options[i].hdr_out_str)+1);
                      else
                          table_set(r->headers_out,options[i].hdr_out_str,"1");
                      break;
                  case STAT_OPT_AUTO:
                      r->content_type = "text/plain";
                      short_report = 1;
                      break;
                }
            }
            i++;
        }
    }

    soft_timeout("send rewrite status info", r);
    send_http_header(r);

    if (r->header_only) 
        return 0;

    rputs("<html>\n", r);
    rputs("<head>\n", r);
    rprintf(r, "<title>Apache Server %s: URL Rewriting Engine Status</title>\n", server->server_hostname);
    rputs("</head>\n",r);
    rputs("<body>\n", r);
    rputs("<center>\n", r);
    rprintf(r, "<h2>Apache Server %s</h2>\n", server->server_hostname);
    rputs("<h1>URL Rewriting Engine Status</h1>\n", r);
    rputs("</center>\n", r);

    rputs("<center>\n", r);
    rvputs(r,"Current Time: ", asctime(localtime(&nowtime)), "<br>\n", NULL);
    rvputs(r,"Restart Time: ", asctime(localtime(&restart_time)), "<br>\n", NULL);
    rputs("</center>\n", r);

    rputs("<hr>\n", r);
    rputs("<p>\n", r);


    sconf = r->server->module_config;
    conf  = (rewrite_server_conf *)get_module_config(sconf, &rewrite_module);
    rewriterules = conf->rewriterules;
    entries = (rewriterule_entry *)rewriterules->elts;

    rputs("<center>\n", r);
    rputs("<h2>Rewrite Rules</h2>\n", r);
    rputs("<p>\n", r);
    rputs("<font size=-2>\n", r);
    rputs("<table cellspacing=0 cellpadding=0>\n", r);
    rputs("<tr>\n", r);
    rputs("<td align=right> </td><td><b>Pattern (Regular Expression)</b></td><td><b>Output (Substitution String)</b></td> <td><b>Flags</b></td>\n", r);
    rputs("</tr>\n", r);
    rputs("<tr>\n", r);
    rputs("<td><hr></td><td><hr></td><td><hr></td><td><hr></td>\n", r);
    rputs("</tr>\n", r);

    for (i = 0; i < rewriterules->nelts; i++) {
        p = &entries[i];
        strcpy(flagstr, "");
        if (p->flags & RULEFLAG_FORCEREDIRECT)
            strcat(flagstr, "force-redirect, ");
        if (p->flags & RULEFLAG_LASTRULE)
            strcat(flagstr, "last, ");
        if (p->flags & RULEFLAG_NEWROUND)
            strcat(flagstr, "next, ");
        if (p->flags & RULEFLAG_CHAIN)
            strcat(flagstr, "chain, ");
        if (p->flags & RULEFLAG_IGNOREONSUBREQ)
            strcat(flagstr, "nosubreq, ");
        if (p->flags & RULEFLAG_PROXY)
            strcat(flagstr, "proxy, ");
        if (p->flags & RULEFLAG_PASSTHROUGH)
            strcat(flagstr, "passthrough, ");
        if (p->flags & RULEFLAG_IGNOREONSUBREQ)
            strcat(flagstr, "nosubreq, ");
        if (p->flags & RULEFLAG_NOTMATCH)
            strcat(notstr, "!");
        if (p->forced_mimetype) {
            strcat(flagstr, "type=");
            strcat(flagstr, p->forced_mimetype);
            strcat(flagstr, ", ");
        }
        if (*flagstr == '\0') 
            strcpy(flagstr, "<em>none</em>");
        else 
            flagstr[strlen(flagstr)-2] = '\0';
        rputs("<tr>\n", r);

        strcpy(output, p->output);
        for (j = 0; output[j] != '\0'; j++) {
            if (output[j] == '\\' && output[j+1] >= '1' && output[j+1] <= '9') 
                output[j] = '$';
        }

        rprintf(r, "<td align=right><b>%d. </b></td> <td><font color=\"#b02020\"><tt>%s%s </tt></font></td> <td><font color=\"#202090\"><tt>%s </tt></font></td><td>%s</td>\n", i+1, notstr, p->pattern, output, flagstr);
        rputs("</tr>\n", r);
    }

    rputs("</table>\n", r);
    rputs("</font>\n", r);
    rputs("</center>\n", r);

#if 0
    rputs("<center>\n", r);
    rputs("<h2>Rewrite Maps</h2>\n", r);
    rputs("</center>\n", r);
    rputs("<p>\n", r);

    rputs("<center>\n", r);
    rputs("<h2>Rewrite Engine Status</h2>\n", r);
    rputs("</center>\n", r);
    rputs("<p>\n", r);

    rputs("<center>\n", r);
    rputs("<h2>Rewrite Logfile</h2>\n", r);
    rputs("</center>\n", r);
    rputs("<p>\n", r);
#endif

    rputs("</body>\n", r);
    rputs("</html>\n", r);
    return 0;
}


/*EOF*/
